@@ -181,6 +181,8 @@ RESERVED_KEYWORDS = {
     "empty_bytes32",
     "max_int128",
     "min_int128",
+    "max_int256",
+    "min_int256",
     "max_decimal",
     "min_decimal",
     "max_uint256",
