@@ -1,4 +1,4 @@
-from typing import Tuple
+from typing import Optional, Tuple
 
 from vyper import ast as vy_ast
 from vyper.context import validation
@@ -8,6 +8,7 @@ from vyper.context.types.bases import (
     DataLocation,
     IndexableTypeDefinition,
 )
+from vyper.context.types.value.numeric import Uint256Definition
 from vyper.exceptions import ArrayIndexException, InvalidType
 
 
@@ -42,6 +43,13 @@ class _SequenceDefinition(IndexableTypeDefinition):
         )
         self.length = length
 
+    def get_signature(self) -> Tuple[Tuple, Optional[BaseTypeDefinition]]:
+        # override the default behavior to return `Uint256Definition`
+        # an external interface cannot use `IntegerAbstractType` because
+        # abstract types have no canonical type
+        new_args, return_type = self.value_type.get_signature()
+        return (Uint256Definition(),) + new_args, return_type
+
 
 class ArrayDefinition(_SequenceDefinition):
     """
@@ -66,10 +74,18 @@ class ArrayDefinition(_SequenceDefinition):
     def __repr__(self):
         return f"{self.value_type}[{self.length}]"
 
+    @property
+    def canonical_type(self):
+        return f"{self.value_type.canonical_type}[{self.length}]"
+
     @property
     def is_dynamic_size(self):
         return self.value_type.is_dynamic_size
 
+    @property
+    def size_in_bytes(self):
+        return self.value_type.size_in_bytes * self.length
+
     def get_index_type(self, node):
         if isinstance(node, vy_ast.Int):
             if node.value < 0:
@@ -113,7 +129,11 @@ class TupleDefinition(_SequenceDefinition):
 
     @property
     def is_dynamic_size(self):
-        return any(i for i in self.self.value_type if i.is_dynamic_size)
+        return any(i for i in self.value_type if i.is_dynamic_size)
+
+    @property
+    def size_in_bytes(self):
+        return sum(i.size_in_bytes for i in self.value_type)
 
     def get_index_type(self, node):
         if not isinstance(node, vy_ast.Int):
