@@ -1,42 +1,31 @@
 from vyper import ast as vy_ast
-from vyper.exceptions import TypeMismatch
+from vyper.context.types.abstract import ArrayValueAbstractType
+from vyper.context.types.indexable.sequence import ArrayDefinition
+from vyper.context.types.value.numeric import Uint256Definition
+from vyper.exceptions import StructureException
 from vyper.parser.expr import Expr
 from vyper.parser.lll_node import LLLnode
 from vyper.parser.parser_utils import (
-    base_type_conversion,
     getpos,
     make_byte_array_copier,
     make_setter,
+    mzero,
     unwrap_location,
     zero_pad,
 )
-from vyper.types.types import (
-    BaseType,
-    ByteArrayLike,
-    ListType,
-    get_size_of_type,
-)
-from vyper.utils import bytes_to_int, ceil32, keccak256
+from vyper.types.types import BaseType, ByteArrayLike
+from vyper.utils import bytes_to_int, keccak256
 
 
-def pack_logging_topics(event_id, args, expected_topics, context, pos):
+def pack_logging_topics(event_id, arg_nodes, arg_types, context):
     topics = [event_id]
-    code_pos = pos
-    for pos, expected_topic in enumerate(expected_topics):
-        expected_type = expected_topic.typ
-        arg = args[pos]
-        value = Expr(arg, context).lll_node
-        arg_type = value.typ
+    for node, typ in zip(arg_nodes, arg_types):
+        value = Expr(node, context).lll_node
 
-        if isinstance(arg_type, ByteArrayLike) and isinstance(expected_type, ByteArrayLike):
-            if arg_type.maxlen > expected_type.maxlen:
-                raise TypeMismatch(
-                    f"Topic input bytes are too big: {arg_type} {expected_type}", code_pos
-                )
-
-            if isinstance(arg, (vy_ast.Str, vy_ast.Bytes)):
+        if isinstance(typ, ArrayValueAbstractType):
+            if isinstance(node, (vy_ast.Str, vy_ast.Bytes)):
                 # for literals, generate the topic at compile time
-                value = arg.value
+                value = node.value
                 if isinstance(value, str):
                     value = value.encode()
                 topics.append(bytes_to_int(keccak256(value)))
@@ -46,7 +35,7 @@ def pack_logging_topics(event_id, args, expected_topics, context, pos):
 
             else:
                 # storage or calldata
-                placeholder = context.new_placeholder(value.typ)
+                placeholder = context.new_internal_variable(value.typ)
                 placeholder_node = LLLnode.from_list(placeholder, typ=value.typ, location="memory")
                 copier = make_byte_array_copier(
                     placeholder_node,
@@ -60,27 +49,21 @@ def pack_logging_topics(event_id, args, expected_topics, context, pos):
                 ]
                 topics.append(lll_node)
 
-        elif isinstance(arg_type, ListType) and isinstance(expected_type, ListType):
-            size = get_size_of_type(value.typ) * 32
+        elif isinstance(typ, ArrayDefinition):
+            size = typ.size_in_bytes
             if value.location == "memory":
                 topics.append(["sha3", value, size])
 
             else:
                 # storage or calldata
-                placeholder = context.new_placeholder(value.typ)
+                placeholder = context.new_internal_variable(value.typ)
                 placeholder_node = LLLnode.from_list(placeholder, typ=value.typ, location="memory")
                 setter = make_setter(placeholder_node, value, "memory", value.pos)
                 lll_node = ["seq", setter, ["sha3", placeholder, size]]
                 topics.append(lll_node)
 
         else:
-            if arg_type != expected_type:
-                raise TypeMismatch(
-                    f"Invalid type for logging topic, got {arg_type} expected {expected_type}",
-                    value.pos,
-                )
             value = unwrap_location(value)
-            value = base_type_conversion(value, arg_type, expected_type, pos=code_pos)
             topics.append(value)
 
     return topics
@@ -110,14 +93,14 @@ def pack_args_by_32(
     :param datamem_start: position where the whole datemem section starts.
     """
 
-    if isinstance(typ, BaseType):
+    if typ.size_in_bytes == 32:
         if isinstance(arg, LLLnode):
             value = unwrap_location(arg)
         else:
             value = Expr(arg, context).lll_node
-            value = base_type_conversion(value, value.typ, typ, pos)
-        holder.append(LLLnode.from_list(["mstore", placeholder, value], typ=typ, location="memory"))
-    elif isinstance(typ, ByteArrayLike):
+            value = unwrap_location(value)
+        holder.append(LLLnode.from_list(["mstore", placeholder, value], location="memory"))
+    elif isinstance(typ, ArrayValueAbstractType):
 
         if isinstance(arg, LLLnode):  # Is prealloacted variable.
             source_lll = arg
@@ -126,10 +109,11 @@ def pack_args_by_32(
 
         # Set static offset, in arg slot.
         holder.append(LLLnode.from_list(["mstore", placeholder, ["mload", dynamic_offset_counter]]))
-        # Get the biginning to write the ByteArray to.
+        # Get the beginning to write the ByteArray to.
+        # TODO refactor out the use of `ByteArrayLike` once old types are removed from parser
         dest_placeholder = LLLnode.from_list(
             ["add", datamem_start, ["mload", dynamic_offset_counter]],
-            typ=typ,
+            typ=ByteArrayLike(typ.length),
             location="memory",
             annotation="pack_args_by_32:dest_placeholder",
         )
@@ -156,15 +140,16 @@ def pack_args_by_32(
             annotation="Increment dynamic offset counter",
         )
         holder.append(increment_counter)
-    elif isinstance(typ, ListType):
-        maxlen += (typ.count - 1) * 32
-        typ = typ.subtype
+    elif isinstance(typ, ArrayDefinition):
 
-        def check_list_type_match(provided):  # Check list types match.
-            if provided != typ:
-                raise TypeMismatch(
-                    f"Log list type '{provided}' does not match provided, expected '{typ}'"
-                )
+        if isinstance(arg, vy_ast.Call) and arg.func.get("id") == "empty":
+            # special case for `empty()` with a static-sized array
+            holder.append(mzero(placeholder, typ.size_in_bytes))
+            maxlen += typ.size_in_bytes - 32
+            return holder, maxlen
+
+        maxlen += (typ.length - 1) * 32
+        typ = typ.value_type
 
         # NOTE: Below code could be refactored into iterators/getter functions for each type of
         #       repetitive loop. But seeing how each one is a unique for loop, and in which way
@@ -173,34 +158,29 @@ def pack_args_by_32(
         # List from storage
         if isinstance(arg, vy_ast.Attribute) and arg.value.id == "self":
             stor_list = context.globals[arg.attr]
-            check_list_type_match(stor_list.typ.subtype)
             size = stor_list.typ.count
             mem_offset = 0
             for i in range(0, size):
                 storage_offset = i
                 arg2 = LLLnode.from_list(
                     ["sload", ["add", ["sha3_32", Expr(arg, context).lll_node], storage_offset]],
-                    typ=typ,
                 )
                 holder, maxlen = pack_args_by_32(
                     holder, maxlen, arg2, typ, context, placeholder + mem_offset, pos=pos,
                 )
-                mem_offset += get_size_of_type(typ) * 32
+                mem_offset += typ.size_in_bytes
 
         # List from variable.
         elif isinstance(arg, vy_ast.Name):
             size = context.vars[arg.id].size
             pos = context.vars[arg.id].pos
-            check_list_type_match(context.vars[arg.id].typ.subtype)
             mem_offset = 0
             for _ in range(0, size):
-                arg2 = LLLnode.from_list(
-                    pos + mem_offset, typ=typ, location=context.vars[arg.id].location
-                )
+                arg2 = LLLnode.from_list(pos + mem_offset, location=context.vars[arg.id].location)
                 holder, maxlen = pack_args_by_32(
                     holder, maxlen, arg2, typ, context, placeholder + mem_offset, pos=pos,
                 )
-                mem_offset += get_size_of_type(typ) * 32
+                mem_offset += typ.size_in_bytes
 
         # List from list literal.
         else:
@@ -209,74 +189,60 @@ def pack_args_by_32(
                 holder, maxlen = pack_args_by_32(
                     holder, maxlen, arg2, typ, context, placeholder + mem_offset, pos=pos,
                 )
-                mem_offset += get_size_of_type(typ) * 32
+                mem_offset += typ.size_in_bytes
     return holder, maxlen
 
 
 # Pack logging data arguments
-def pack_logging_data(expected_data, args, context, pos):
+def pack_logging_data(arg_nodes, arg_types, context, pos):
     # Checks to see if there's any data
-    if not args:
+    if not arg_nodes:
         return ["seq"], 0, None, 0
     holder = ["seq"]
-    maxlen = len(args) * 32  # total size of all packed args (upper limit)
+    maxlen = len(arg_nodes) * 32  # total size of all packed args (upper limit)
 
     # Unroll any function calls, to temp variables.
     prealloacted = {}
-    for idx, (arg, _expected_arg) in enumerate(zip(args, expected_data)):
+    for idx, node in enumerate(arg_nodes):
 
-        if isinstance(arg, (vy_ast.Str, vy_ast.Call)):
-            expr = Expr(arg, context)
+        if isinstance(node, (vy_ast.Str, vy_ast.Call)) and node.get("func.id") != "empty":
+            expr = Expr(node, context)
             source_lll = expr.lll_node
-            typ = source_lll.typ
-
-            if isinstance(arg, vy_ast.Str):
-                if len(arg.s) > typ.maxlen:
-                    raise TypeMismatch(f"Data input bytes are to big: {len(arg.s)} {typ}", pos)
-
-            tmp_variable = context.new_internal_variable(
-                f"_log_pack_var_{arg.lineno}_{arg.col_offset}", source_lll.typ,
-            )
+            tmp_variable = context.new_internal_variable(source_lll.typ)
             tmp_variable_node = LLLnode.from_list(
                 tmp_variable,
                 typ=source_lll.typ,
-                pos=getpos(arg),
+                pos=getpos(node),
                 location="memory",
                 annotation=f"log_prealloacted {source_lll.typ}",
             )
-            # Store len.
-            # holder.append(['mstore', len_placeholder, ['mload', unwrap_location(source_lll)]])
             # Copy bytes.
-
             holder.append(
-                make_setter(tmp_variable_node, source_lll, pos=getpos(arg), location="memory")
+                make_setter(tmp_variable_node, source_lll, pos=getpos(node), location="memory")
             )
             prealloacted[idx] = tmp_variable_node
 
-    requires_dynamic_offset = any([isinstance(data.typ, ByteArrayLike) for data in expected_data])
-    if requires_dynamic_offset:
-        dynamic_offset_counter = context.new_placeholder(BaseType(32))
-        dynamic_placeholder = context.new_placeholder(BaseType(32))
-    else:
-        dynamic_offset_counter = None
+    # Create internal variables for for dynamic and static args.
+    static_types = []
+    for typ in arg_types:
+        static_types.append(typ if not typ.is_dynamic_size else Uint256Definition())
 
-    # Create placeholder for static args. Note: order of new_*() is important.
-    placeholder_map = {}
-    for i, (_arg, data) in enumerate(zip(args, expected_data)):
-        typ = data.typ
-        if not isinstance(typ, ByteArrayLike):
-            placeholder = context.new_placeholder(typ)
-        else:
-            placeholder = context.new_placeholder(BaseType(32))
-        placeholder_map[i] = placeholder
+    requires_dynamic_offset = any(typ.is_dynamic_size for typ in arg_types)
+
+    dynamic_offset_counter = None
+    if requires_dynamic_offset:
+        # TODO refactor out old type objects
+        dynamic_offset_counter = context.new_internal_variable(BaseType(32))
+        dynamic_placeholder = context.new_internal_variable(BaseType(32))
+
+    static_vars = [context.new_internal_variable(i) for i in static_types]
 
     # Populate static placeholders.
-    for i, (arg, data) in enumerate(zip(args, expected_data)):
-        typ = data.typ
-        placeholder = placeholder_map[i]
-        if not isinstance(typ, ByteArrayLike):
+    for i, (node, typ) in enumerate(zip(arg_nodes, arg_types)):
+        placeholder = static_vars[i]
+        if not isinstance(typ, ArrayValueAbstractType):
             holder, maxlen = pack_args_by_32(
-                holder, maxlen, prealloacted.get(i, arg), typ, context, placeholder, pos=pos,
+                holder, maxlen, prealloacted.get(i, node), typ, context, placeholder, pos=pos,
             )
 
     # Dynamic position starts right after the static args.
@@ -284,27 +250,30 @@ def pack_logging_data(expected_data, args, context, pos):
         holder.append(LLLnode.from_list(["mstore", dynamic_offset_counter, maxlen]))
 
     # Calculate maximum dynamic offset placeholders, used for gas estimation.
-    for _arg, data in zip(args, expected_data):
-        typ = data.typ
-        if isinstance(typ, ByteArrayLike):
-            maxlen += 32 + ceil32(typ.maxlen)
+    for typ in arg_types:
+        if typ.is_dynamic_size:
+            maxlen += typ.size_in_bytes
 
     if requires_dynamic_offset:
         datamem_start = dynamic_placeholder + 32
     else:
-        datamem_start = placeholder_map[0]
+        datamem_start = static_vars[0]
 
     # Copy necessary data into allocated dynamic section.
-    for i, (arg, data) in enumerate(zip(args, expected_data)):
-        typ = data.typ
-        if isinstance(typ, ByteArrayLike):
+    for i, (node, typ) in enumerate(zip(arg_nodes, arg_types)):
+        if isinstance(typ, ArrayValueAbstractType):
+            if isinstance(node, vy_ast.Call) and node.func.get("id") == "empty":
+                # TODO add support for this
+                raise StructureException(
+                    "Cannot use `empty` on Bytes or String types within an event log", node
+                )
             pack_args_by_32(
                 holder=holder,
                 maxlen=maxlen,
-                arg=prealloacted.get(i, arg),
+                arg=prealloacted.get(i, node),
                 typ=typ,
                 context=context,
-                placeholder=placeholder_map[i],
+                placeholder=static_vars[i],
                 datamem_start=datamem_start,
                 dynamic_offset_counter=dynamic_offset_counter,
                 pos=pos,
