@@ -48,8 +48,10 @@ from vyper.utils import (
 BUILTIN_CONSTANTS = {
     "EMPTY_BYTES32": (0, "bytes32"),
     "ZERO_ADDRESS": (0, "address"),
-    "MAX_INT128": (SizeLimits.MAXNUM, "int128"),
-    "MIN_INT128": (SizeLimits.MINNUM, "int128"),
+    "MAX_INT128": (SizeLimits.MAX_INT128, "int128"),
+    "MIN_INT128": (SizeLimits.MIN_INT128, "int128"),
+    "MAX_INT256": (SizeLimits.MAX_INT256, "int256"),
+    "MIN_INT256": (SizeLimits.MIN_INT256, "int256"),
     "MAX_DECIMAL": (SizeLimits.MAXDECIMAL, "decimal"),
     "MIN_DECIMAL": (SizeLimits.MINDECIMAL, "decimal"),
     "MAX_UINT256": (SizeLimits.MAX_UINT256, "uint256"),
@@ -203,10 +205,10 @@ class Expr:
             raise TypeCheckFailure(f"{type(node).__name__} node did not produce LLL")
 
     def parse_Int(self):
-        # Literal (mostly likely) becomes int128
-        if SizeLimits.in_bounds("int128", self.expr.n) or self.expr.n < 0:
+        # Literal (mostly likely) becomes int256
+        if self.expr.n < 0:
             return LLLnode.from_list(
-                self.expr.n, typ=BaseType("int128", is_literal=True), pos=getpos(self.expr),
+                self.expr.n, typ=BaseType("int256", is_literal=True), pos=getpos(self.expr),
             )
         # Literal is large enough (mostly likely) becomes uint256.
         else:
@@ -216,7 +218,7 @@ class Expr:
 
     def parse_Decimal(self):
         numstring, num, den = get_number_as_fraction(self.expr, self.context)
-        if not (SizeLimits.MINNUM * den <= num <= SizeLimits.MAXNUM * den):
+        if not (SizeLimits.MIN_INT128 * den <= num <= SizeLimits.MAX_INT128 * den):
             return
         if DECIMAL_DIVISOR % den:
             return
@@ -371,6 +373,26 @@ class Expr:
             key = f"{self.expr.value.id}.{self.expr.attr}"
             if key == "msg.sender" and not self.context.is_internal:
                 return LLLnode.from_list(["caller"], typ="address", pos=getpos(self.expr))
+            elif key == "msg.data" and not self.context.is_internal:
+                is_len = self.expr._metadata.get("is_len")
+                if is_len is True:
+                    typ = ByteArrayType(32)
+                    pos = self.context.new_internal_variable(typ)
+                    node = ["seq", ["mstore", pos, "calldatasize"], pos]
+                    return LLLnode.from_list(
+                        node, typ=typ, pos=getpos(self.expr), location="memory"
+                    )
+                size = self.expr._metadata.get("size")
+                typ = ByteArrayType(size + 32)
+                pos = self.context.new_internal_variable(typ)
+                node = [
+                    "seq",
+                    ["assert", ["le", size, "calldatasize"]],
+                    ["mstore", pos, size],
+                    ["calldatacopy", pos + 32, 0, size],
+                    pos,
+                ]
+                return LLLnode.from_list(node, typ=typ, pos=getpos(self.expr), location="memory")
             elif key == "msg.value" and self.context.is_payable:
                 return LLLnode.from_list(
                     ["callvalue"], typ=BaseType("uint256"), pos=getpos(self.expr),
@@ -452,50 +474,80 @@ class Expr:
         if not is_numeric_type(left.typ) or not is_numeric_type(right.typ):
             return
 
-        arithmetic_pair = {left.typ.typ, right.typ.typ}
         pos = getpos(self.expr)
+        types = {left.typ.typ, right.typ.typ}
+        literals = {left.typ.is_literal, right.typ.is_literal}
 
-        # Special case with uint256 were int literal may be casted.
-        if arithmetic_pair == {"uint256", "int128"}:
-            # Check right side literal.
-            if right.typ.is_literal and SizeLimits.in_bounds("uint256", right.value):
-                right = LLLnode.from_list(
-                    right.value, typ=BaseType("uint256", None, is_literal=True), pos=pos,
-                )
-
-            # Check left side literal.
-            elif left.typ.is_literal and SizeLimits.in_bounds("uint256", left.value):
+        # If one value of the operation is a literal, we recast it to match the non-literal type.
+        # We know this is OK because types were already verified in the actual typechecking pass.
+        # This is a temporary solution to not break parser while we work toward removing types
+        # altogether at this stage of complition. @iamdefinitelyahuman
+        if literals == {True, False} and len(types) > 1 and "decimal" not in types:
+            if left.typ.is_literal and SizeLimits.in_bounds(right.typ.typ, left.value):
                 left = LLLnode.from_list(
-                    left.value, typ=BaseType("uint256", None, is_literal=True), pos=pos,
+                    left.value, typ=BaseType(right.typ.typ, None, is_literal=True), pos=pos,
+                )
+            elif right.typ.is_literal and SizeLimits.in_bounds(left.typ.typ, right.value):
+                right = LLLnode.from_list(
+                    right.value, typ=BaseType(left.typ.typ, None, is_literal=True), pos=pos,
                 )
-
-        if left.typ.typ == "decimal" and isinstance(self.expr.op, vy_ast.Pow):
-            return
-
-        # Only allow explicit conversions to occur.
-        if left.typ.typ != right.typ.typ:
-            return
 
         ltyp, rtyp = left.typ.typ, right.typ.typ
+        if ltyp != rtyp:
+            # Sanity check - ensure that we aren't dealing with different types
+            # This should be unreachable due to the type check pass
+            return
+
         arith = None
         if isinstance(self.expr.op, (vy_ast.Add, vy_ast.Sub)):
             new_typ = BaseType(ltyp)
-            op = "add" if isinstance(self.expr.op, vy_ast.Add) else "sub"
 
-            if ltyp == "uint256" and isinstance(self.expr.op, vy_ast.Add):
-                # safeadd
-                arith = ["seq", ["assert", ["ge", ["add", "l", "r"], "l"]], ["add", "l", "r"]]
+            if ltyp == "uint256":
+                if isinstance(self.expr.op, vy_ast.Add):
+                    # safeadd
+                    arith = ["seq", ["assert", ["ge", ["add", "l", "r"], "l"]], ["add", "l", "r"]]
 
-            elif ltyp == "uint256" and isinstance(self.expr.op, vy_ast.Sub):
-                # safesub
-                arith = ["seq", ["assert", ["ge", "l", "r"]], ["sub", "l", "r"]]
+                elif isinstance(self.expr.op, vy_ast.Sub):
+                    # safesub
+                    arith = ["seq", ["assert", ["ge", "l", "r"]], ["sub", "l", "r"]]
 
-            elif ltyp == rtyp:
+            elif ltyp == "int256":
+                if isinstance(self.expr.op, vy_ast.Add):
+                    op, comp1, comp2 = "add", "sge", "slt"
+                else:
+                    op, comp1, comp2 = "sub", "sle", "sgt"
+
+                if right.typ.is_literal:
+                    if right.value >= 0:
+                        arith = ["seq", ["assert", [comp1, [op, "l", "r"], "l"]], [op, "l", "r"]]
+                    else:
+                        arith = ["seq", ["assert", [comp2, [op, "l", "r"], "l"]], [op, "l", "r"]]
+                else:
+                    arith = [
+                        "with",
+                        "ans",
+                        [op, "l", "r"],
+                        [
+                            "seq",
+                            [
+                                "assert",
+                                [
+                                    "or",
+                                    ["and", ["sge", "r", 0], [comp1, "ans", "l"]],
+                                    ["and", ["slt", "r", 0], [comp2, "ans", "l"]],
+                                ],
+                            ],
+                            "ans",
+                        ],
+                    ]
+
+            elif ltyp in ("decimal", "int128"):
+                op = "add" if isinstance(self.expr.op, vy_ast.Add) else "sub"
                 arith = [op, "l", "r"]
 
         elif isinstance(self.expr.op, vy_ast.Mult):
             new_typ = BaseType(ltyp)
-            if ltyp == rtyp == "uint256":
+            if ltyp == "uint256":
                 arith = [
                     "with",
                     "ans",
@@ -507,12 +559,38 @@ class Expr:
                     ],
                 ]
 
-            elif ltyp == rtyp == "int128":
-                # TODO should this be 'smul' (note edge cases in YP for smul)
+            elif ltyp == "int256":
+                if version_check(begin="constantinople"):
+                    upper_bound = ["shl", 255, 1]
+                else:
+                    upper_bound = -(2 ** 255)
+                if not left.typ.is_literal and not right.typ.is_literal:
+                    bounds_check = [
+                        "assert",
+                        ["or", ["ne", "l", ["not", 0]], ["ne", "r", upper_bound]],
+                    ]
+                elif left.typ.is_literal and left.value == -1:
+                    bounds_check = ["assert", ["ne", "r", upper_bound]]
+                elif right.typ.is_literal and right.value == -(2 ** 255):
+                    bounds_check = ["assert", ["ne", "l", ["not", 0]]]
+                else:
+                    bounds_check = "pass"
+                arith = [
+                    "with",
+                    "ans",
+                    ["mul", "l", "r"],
+                    [
+                        "seq",
+                        bounds_check,
+                        ["assert", ["or", ["eq", ["sdiv", "ans", "l"], "r"], ["iszero", "l"]]],
+                        "ans",
+                    ],
+                ]
+
+            elif ltyp == "int128":
                 arith = ["mul", "l", "r"]
 
-            elif ltyp == rtyp == "decimal":
-                # TODO should this be smul
+            elif ltyp == "decimal":
                 arith = [
                     "with",
                     "ans",
@@ -536,16 +614,33 @@ class Expr:
                 # only apply the non-zero clamp when r is not a constant
                 divisor = ["clamp_nonzero", "r"]
 
-            if ltyp == rtyp == "uint256":
+            if ltyp == "uint256":
                 arith = ["div", "l", divisor]
 
-            elif ltyp == rtyp == "int128":
+            elif ltyp == "int256":
+                if version_check(begin="constantinople"):
+                    upper_bound = ["shl", 255, 1]
+                else:
+                    upper_bound = -(2 ** 255)
+                if not left.typ.is_literal and not right.typ.is_literal:
+                    bounds_check = [
+                        "assert",
+                        ["or", ["ne", "r", ["not", 0]], ["ne", "l", upper_bound]],
+                    ]
+                elif left.typ.is_literal and left.value == -(2 ** 255):
+                    bounds_check = ["assert", ["ne", "r", ["not", 0]]]
+                elif right.typ.is_literal and right.value == -1:
+                    bounds_check = ["assert", ["ne", "l", upper_bound]]
+                else:
+                    bounds_check = "pass"
+                arith = ["seq", bounds_check, ["sdiv", "l", divisor]]
+
+            elif ltyp in ("int128", "int256"):
                 arith = ["sdiv", "l", divisor]
 
-            elif ltyp == rtyp == "decimal":
+            elif ltyp == "decimal":
                 arith = [
                     "sdiv",
-                    # TODO check overflow cases, also should it be smul
                     ["mul", "l", DECIMAL_DIVISOR],
                     divisor,
                 ]
@@ -562,15 +657,12 @@ class Expr:
                 # only apply the non-zero clamp when r is not a constant
                 divisor = ["clamp_nonzero", "r"]
 
-            if ltyp == rtyp == "uint256":
+            if ltyp == "uint256":
                 arith = ["mod", "l", divisor]
-            elif ltyp == rtyp:
-                # TODO should this be regular mod
+            else:
                 arith = ["smod", "l", divisor]
 
         elif isinstance(self.expr.op, vy_ast.Pow):
-            if ltyp != "int128" and ltyp != "uint256" and isinstance(self.expr.right, vy_ast.Name):
-                return
             new_typ = BaseType(ltyp)
 
             if self.expr.left.get("value") == 1:
@@ -581,6 +673,9 @@ class Expr:
             if ltyp == "int128":
                 is_signed = True
                 num_bits = 128
+            elif ltyp == "int256":
+                is_signed = True
+                num_bits = 256
             else:
                 is_signed = False
                 num_bits = 256
@@ -624,7 +719,7 @@ class Expr:
                     ["mload", MemoryPositions.MAXDECIMAL],
                 ]
             )
-        elif new_typ.typ == "uint256":
+        elif new_typ.typ in ("uint256", "int256"):
             p.append(arith)
         else:
             return
@@ -731,8 +826,6 @@ class Expr:
         elif isinstance(self.expr.op, (vy_ast.In, vy_ast.NotIn)) and isinstance(
             right.typ, ListType
         ):
-            if left.typ != right.typ.subtype:
-                return
             return self.build_in_comparator()
 
         if isinstance(self.expr.op, vy_ast.Gt):
@@ -785,39 +878,16 @@ class Expr:
                 )
 
         # Compare other types.
-        if not is_numeric_type(left.typ) or not is_numeric_type(right.typ):
-            if op not in ("eq", "ne"):
-                return
-        left_type, right_type = left.typ.typ, right.typ.typ
+        if is_numeric_type(left.typ) and is_numeric_type(right.typ):
+            if left.typ.typ == right.typ.typ == "uint256":
+                # this works because we only have one unsigned integer type
+                # in the future if others are added, this logic must be expanded
+                op = self._signed_to_unsigned_comparision_op(op)
 
-        # Special Case: comparison of a literal integer. If in valid range allow it to be compared.
-        if {left_type, right_type} == {"int128", "uint256"} and {
-            left.typ.is_literal,
-            right.typ.is_literal,
-        } == {
-            True,
-            False,
-        }:  # noqa: E501
-
-            comparison_allowed = False
-            if left.typ.is_literal and SizeLimits.in_bounds(right_type, left.value):
-                comparison_allowed = True
-            elif right.typ.is_literal and SizeLimits.in_bounds(left_type, right.value):
-                comparison_allowed = True
-            op = self._signed_to_unsigned_comparision_op(op)
-
-            if comparison_allowed:
-                return LLLnode.from_list([op, left, right], typ="bool", pos=getpos(self.expr))
-
-        elif {left_type, right_type} == {"uint256", "uint256"}:
-            op = self._signed_to_unsigned_comparision_op(op)
-        elif (
-            left_type in ("decimal", "int128") or right_type in ("decimal", "int128")
-        ) and left_type != right_type:  # noqa: E501
+        elif op not in ("eq", "ne"):
             return
 
-        if left_type == right_type:
-            return LLLnode.from_list([op, left, right], typ="bool", pos=getpos(self.expr))
+        return LLLnode.from_list([op, left, right], typ="bool", pos=getpos(self.expr))
 
     def parse_BoolOp(self):
         for value in self.expr.values:
@@ -929,28 +999,25 @@ class Expr:
             return external_call.make_external_call(self.expr, self.context)
 
     def parse_List(self):
-        if not len(self.expr.elements):
-            return
+        call_lll, multi_lll = parse_sequence(self.expr, self.expr.elements, self.context)
+        out_type = next((i.typ for i in multi_lll if not i.typ.is_literal), multi_lll[0].typ)
+        typ = ListType(out_type, len(self.expr.elements), is_literal=True)
+        multi_lll = LLLnode.from_list(["multi"] + multi_lll, typ=typ, pos=getpos(self.expr))
+        if not call_lll:
+            return multi_lll
 
-        def get_out_type(lll_node):
-            if isinstance(lll_node, ListType):
-                return get_out_type(lll_node.subtype)
-            return lll_node.typ
+        lll_node = ["seq_unchecked"] + call_lll + [multi_lll]
+        return LLLnode.from_list(lll_node, typ=typ, pos=getpos(self.expr))
 
-        lll_node = []
-        out_type = None
+    def parse_Tuple(self):
+        call_lll, multi_lll = parse_sequence(self.expr, self.expr.elements, self.context)
+        typ = TupleType([x.typ for x in multi_lll], is_literal=True)
+        multi_lll = LLLnode.from_list(["multi"] + multi_lll, typ=typ, pos=getpos(self.expr))
+        if not call_lll:
+            return multi_lll
 
-        for elt in self.expr.elements:
-            current_lll_node = Expr(elt, self.context).lll_node
-            if not out_type or not current_lll_node.typ.is_literal:
-                # prefer to use a non-literal type here, because literals can be ambiguous
-                # this should be removed altogether as we refactor types out of parser
-                out_type = current_lll_node.typ
-            lll_node.append(current_lll_node)
-
-        return LLLnode.from_list(
-            ["multi"] + lll_node, typ=ListType(out_type, len(lll_node)), pos=getpos(self.expr),
-        )
+        lll_node = ["seq_unchecked"] + call_lll + [multi_lll]
+        return LLLnode.from_list(lll_node, typ=typ, pos=getpos(self.expr))
 
     @staticmethod
     def struct_literals(expr, name, context):
@@ -970,39 +1037,6 @@ class Expr:
             pos=getpos(expr),
         )
 
-    def parse_Tuple(self):
-        if not len(self.expr.elements):
-            return
-        call_lll = []
-        multi_lll = []
-        for node in self.expr.elements:
-            if isinstance(node, vy_ast.Call):
-                # for calls inside the tuple, we perform the call prior to building the tuple and
-                # assign it's result to memory - otherwise there is potential for memory corruption
-                lll_node = Expr(node, self.context).lll_node
-                target = LLLnode.from_list(
-                    self.context.new_internal_variable(lll_node.typ),
-                    typ=lll_node.typ,
-                    location="memory",
-                    pos=getpos(self.expr),
-                )
-                call_lll.append(make_setter(target, lll_node, "memory", pos=getpos(self.expr)))
-                multi_lll.append(
-                    LLLnode.from_list(
-                        target, typ=lll_node.typ, pos=getpos(self.expr), location="memory"
-                    ),
-                )
-            else:
-                multi_lll.append(Expr(node, self.context).lll_node)
-
-        typ = TupleType([x.typ for x in multi_lll], is_literal=True)
-        multi_lll = LLLnode.from_list(["multi"] + multi_lll, typ=typ, pos=getpos(self.expr))
-        if not call_lll:
-            return multi_lll
-
-        lll_node = ["seq_unchecked"] + call_lll + [multi_lll]
-        return LLLnode.from_list(lll_node, typ=typ, pos=getpos(self.expr))
-
     # Parse an expression that results in a value
     @classmethod
     def parse_value_expr(cls, expr, context):
@@ -1015,3 +1049,63 @@ class Expr:
         if not o.location:
             raise StructureException("Looking for a variable location, instead got a value", expr)
         return o
+
+
+def parse_sequence(base_node, elements, context):
+    """
+    Generate an LLL node from a sequence of Vyper AST nodes, such as values inside a
+    list/tuple or arguments inside a call.
+
+    Arguments
+    ---------
+    base_node : VyperNode
+        Parent node which contains the sequence being parsed.
+    elements : List[VyperNode]
+        A list of nodes within the sequence.
+    context : Context
+        Currently active local context.
+
+    Returns
+    -------
+    List[LLLNode]
+        LLL nodes that must execute prior to generating the actual sequence in order to
+        avoid memory corruption issues. This list may be empty, depending on the values
+        within `elements`.
+    List[LLLNode]
+        LLL nodes which collectively represent `elements`.
+    """
+    init_lll = []
+    sequence_lll = []
+    for node in elements:
+        if isinstance(node, vy_ast.List):
+            # for nested lists, ensure the init LLL is also processed before the values
+            init, seq = parse_sequence(node, node.elements, context)
+            init_lll.extend(init)
+            out_type = next((i.typ for i in seq if not i.typ.is_literal), seq[0].typ)
+            typ = ListType(out_type, len(node.elements), is_literal=True)
+            multi_lll = LLLnode.from_list(["multi"] + seq, typ=typ, pos=getpos(node))
+            sequence_lll.append(multi_lll)
+            continue
+
+        lll_node = Expr(node, context).lll_node
+        if isinstance(node, vy_ast.Call) or (
+            isinstance(node, vy_ast.Subscript) and isinstance(node.value, vy_ast.Call)
+        ):
+            # nodes which potentially create their own internal memory variables, and so must
+            # be parsed prior to generating the final sequence to avoid memory corruption
+            target = LLLnode.from_list(
+                context.new_internal_variable(lll_node.typ),
+                typ=lll_node.typ,
+                location="memory",
+                pos=getpos(base_node),
+            )
+            init_lll.append(make_setter(target, lll_node, "memory", pos=getpos(base_node)))
+            sequence_lll.append(
+                LLLnode.from_list(
+                    target, typ=lll_node.typ, pos=getpos(base_node), location="memory"
+                ),
+            )
+        else:
+            sequence_lll.append(lll_node)
+
+    return init_lll, sequence_lll
