@@ -1,7 +1,6 @@
-import ast
 from typing import Any, List, Union
 
-from vyper.exceptions import FunctionDeclarationException
+from vyper import ast as vy_ast
 from vyper.parser.arg_clamps import make_arg_clamper
 from vyper.parser.context import Context, VariableRecord
 from vyper.parser.expr import Expr
@@ -10,7 +9,6 @@ from vyper.parser.function_definitions.utils import (
     get_nonreentrant_lock,
     get_sig_statements,
 )
-from vyper.parser.global_context import GlobalContext
 from vyper.parser.lll_node import LLLnode
 from vyper.parser.parser_utils import getpos, make_setter
 from vyper.parser.stmt import parse_body
@@ -34,42 +32,23 @@ def get_external_arg_copier(
     return copier
 
 
-def validate_external_function(
-    code: ast.FunctionDef, sig: FunctionSignature, global_ctx: GlobalContext
-) -> None:
-    """ Validate external function definition. """
-
-    # __init__ function may not have defaults.
-    if sig.is_initializer() and sig.total_default_args > 0:
-        raise FunctionDeclarationException(
-            "__init__ function may not have default parameters.", code
-        )
-
-    # Check for duplicate variables with globals
-    for arg in sig.args:
-        if arg.name in global_ctx._globals:
-            raise FunctionDeclarationException(
-                "Variable name duplicated between " "function arguments and globals: " + arg.name,
-                code,
-            )
-
-
 def parse_external_function(
-    code: ast.FunctionDef, sig: FunctionSignature, context: Context, is_contract_payable: bool
+    code: vy_ast.FunctionDef, sig: FunctionSignature, context: Context, check_nonpayable: bool,
 ) -> LLLnode:
     """
     Parse a external function (FuncDef), and produce full function body.
 
     :param sig: the FuntionSignature
     :param code: ast of function
-    :param is_contract_payable: bool - does this contract contain payable functions?
+    :param check_nonpayable: if True, include a check that `msg.value == 0`
+                             at the beginning of the function
     :return: full sig compare & function body
     """
 
-    validate_external_function(code, sig, context.global_ctx)
+    func_type = code._metadata["type"]
 
     # Get nonreentrant lock
-    nonreentrant_pre, nonreentrant_post = get_nonreentrant_lock(sig, context.global_ctx)
+    nonreentrant_pre, nonreentrant_post = get_nonreentrant_lock(func_type, context.global_ctx)
 
     clampers = []
 
@@ -79,10 +58,10 @@ def parse_external_function(
         copier = ["pass"]
     elif sig.name == "__init__":
         copier = ["codecopy", MemoryPositions.RESERVED_MEMORY, "~codelen", sig.base_copy_size]
-        context.memory_allocator.increase_memory(sig.max_copy_size)
+        context.memory_allocator.expand_memory(sig.max_copy_size)
     clampers.append(copier)
 
-    if is_contract_payable and sig.mutability != "payable":
+    if check_nonpayable and sig.mutability != "payable":
         # if the contract contains payable functions, but this is not one of them
         # add an assertion that the value of the call is zero
         clampers.append(["assert", ["iszero", "callvalue"]])
@@ -100,7 +79,7 @@ def parse_external_function(
                 )
             )
         if isinstance(arg.typ, ByteArrayLike):
-            mem_pos = context.memory_allocator.increase_memory(32 * get_size_of_type(arg.typ))
+            mem_pos = context.memory_allocator.expand_memory(32 * get_size_of_type(arg.typ))
             context.vars[arg.name] = VariableRecord(arg.name, mem_pos, arg.typ, False)
         else:
             if sig.name == "__init__":
@@ -109,7 +88,7 @@ def parse_external_function(
                 )
             elif i >= default_args_start_pos:  # default args need to be allocated in memory.
                 type_size = get_size_of_type(arg.typ) * 32
-                default_arg_pos = context.memory_allocator.increase_memory(type_size)
+                default_arg_pos = context.memory_allocator.expand_memory(type_size)
                 context.vars[arg.name] = VariableRecord(
                     name=arg.name, pos=default_arg_pos, typ=arg.typ, mutable=False,
                 )
@@ -127,12 +106,8 @@ def parse_external_function(
         )
     # Is default function.
     elif sig.is_default_func():
-        if len(sig.args) > 0:
-            raise FunctionDeclarationException(
-                "Default function may not receive any arguments.", code
-            )
         o = LLLnode.from_list(
-            ["seq"] + clampers + [parse_body(code.body, context)],  # type: ignore
+            ["seq"] + clampers + [parse_body(code.body, context)] + [["stop"]],  # type: ignore
             pos=getpos(code),
         )
     # Is a normal function.
@@ -219,23 +194,22 @@ def parse_external_function(
                 )
 
             # Function with default parameters.
+            function_jump_label = f"{sig.name}_{sig.method_id}_skip"
             o = LLLnode.from_list(
                 [
                     "seq",
                     sig_chain,
                     [
-                        "if",
-                        0,  # can only be jumped into
-                        [
-                            "seq",
-                            ["label", function_routine],
-                            ["seq"]
-                            + nonreentrant_pre
-                            + clampers
-                            + [parse_body(c, context) for c in code.body]
-                            + nonreentrant_post
-                            + [["stop"]],
-                        ],
+                        "seq",
+                        ["goto", function_jump_label],
+                        ["label", function_routine],
+                        ["seq"]
+                        + nonreentrant_pre
+                        + clampers
+                        + [parse_body(c, context) for c in code.body]
+                        + nonreentrant_post
+                        + [["stop"]],
+                        ["label", function_jump_label],
                     ],
                 ],
                 typ=None,
